
# Version 1.1
#
#************************************
# Install or loading required packages
#************************************
source("required_libraries.R")

#************************************
# Loading required function
#************************************
source("all_function.R")

#************************************
# Download PISA data with EdSurvey package
#************************************
# Valid years are 2000, 2003, 2006, 2009, 2012, and 2015.
# Files are placed in a folder named PISA/[year].
# for Windows "C:/Users/", for Mac "/Users/"
myroot <- "C:/Users/Laura/Documents/R/EFECTAS/" #write your own path
year <- 2015
downloadPISA(myroot, year)

#************************************
# Preparation of data for statistical analysis
#************************************
# names for plausible values "math", "read", "scie", "scep", "sced", "scid", "skco", "skpe", "ssph", "ssli", "sses", "flit", "clps"
mycountries <- c("LTU")
myvariables <- c("math", "read", "scie","escs", "hisei", "st011q12ta", "st004d01t", "st034q02ta", "sc012q01ta")
mydata <- form_data(myroot, mycountries, myvariables)
summary(mydata) #brief information about each column
colnames(mydata) #column names

#************************************
# Descriptive statistic
#************************************

# Frequency Table and percentage for categorical variables

# 1 example
dsc_variables1 <-  c("st011q12ta", "st004d01t", "st034q02ta", "sc012q01ta")
(table1 <- frequency_table(mydata, myvariables, dsc_variables1))

# 2 example
dsc_variables1 <-  c("st011q12ta", "st034q02ta")
dsc_group <- c("st004d01t")
(table2 <- frequency_table(mydata, myvariables, dsc_variables1, mygroup = dsc_group))

# 3 example
dsc_variables1 <-  c("st011q12ta", "st034q02ta", " escs")
dsc_group <- c("st004d01t")
unique = TRUE
(table3 <- frequency_table(mydata, myvariables, dsc_variables1, mygroup = dsc_group, uniq = unique))

# 4 example
# only for one variable!!!
dsc_variables1 <-  c("st011q12ta")
missing_values = TRUE
(table4 <- frequency_table(mydata, myvariables, dsc_variables1, missing_values = missing_values))

# Min, Max, Average, Standard deviation, Median, Quartile for ordinal variables
dsc_variables2 <- c("escs", "pv1scie", "pv2scie", "pv3scie", "pv4scie", "pv5scie", "pv6scie", "pv7scie", "pv8scie",
                "pv9scie", "pv10scie", "scie")
(table5 <- descriptive(dsc_variables2))

# Percentiles
prc_variables <- "escs"
percent <- c(5, 25, 50, 75, 95)
(percentiles <- tile(prc_variables, percent))

# Benchmarks only for all plausible values
bn_variable <- "scie"
bench <- c(357.77, 420.07,482.38, 544.68, 606.99, 669.30)
(marks <- ben_marks(mydata, bn_variable, bench))


#************************************
# Correlation
#************************************

# 1 example
cor_variables <- c("escs","pv1scie", "pv2scie")
(cor_coef <- correlation(mydata, myvariables, cor_variables))

# 2 example
cor_variables <- c("escs","hisei", "scie")
(cor_coef <- correlation(mydata, myvariables, cor_variables))

# 3 example
cor_variables <- c("escs","hisei", "scie")
group <- c("st004d01t")
(cor_coef <- correlation(mydata, myvariables, cor_variables, group = group))

# 4 example
cor_variables <- c("escs", "pv1scie")
method <- "Spearman"
(cor_coef <- correlation(mydata, myvariables, cor_variables, method = method))


#************************************
# Regression
#************************************

# Linear Regression
# 1 example
depended <- "scie" 
independed <- c("st011q12ta", "st004d01t")
package <- 1 #1 - BIFIEsurvey, 2 - EdSurvey, 3 - intsvy
reg_equation <- line_regression(mydata, myvariables, depended, independed, package)
summary(reg_equation)

# 2 example
depended <- "scie" 
independed <- c("st011q12ta", "st004d01t")
package <- 2 #1 - BIFIEsurvey, 2 - EdSurvey, 3 - intsvy
reg_equation <- line_regression(mydata, myvariables, depended, independed, package)
summary(reg_equation)

# 3 example
depended <- "scie" 
independed <- c("st011q12ta", "st004d01t")
package <- 3 #1 - BIFIEsurvey, 2 - EdSurvey, 3 - intsvy
reg_equation <- line_regression(mydata, myvariables, depended, independed, package)
summary(reg_equation)

# Multi Linear Regression
depended <- c("scie","read") 
independed <- c("st004d01t")
reg_equation_multi <- multi_regression(mydata, depended, independed)
summary(reg_equation_multi)

# Logistic Regression
depended <- "scie"
independed <- c("st011q12ta", "st004d01t")
(reg_log_equation <- log_regression(mydata, myvariables, depended, independed))


#************************************
# Multilevel
#************************************

# Data preparation according to OECD manual
change <- c("FEMALE", 1, "MALE", 0)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "st004d01t", newname = "gender", change = change)
change <- c("NO RESPONSE", 9, "OTHER COUNTRY", 2, "COUNTRY OF TEST", 1)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "st019aq01t", change = change)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "st019bq01t", change = change)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "st019cq01t", change = change)
mydata1$immig <- (100*mydata1$st019aq01t)+(10*mydata1$st019bq01t)+(mydata1$st019cq01t)
change <- c(111, 0, 121, 0, 112, 0, 122, 1, 222, 1)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "immig", change = change)
mydata1$st019aq01t <- NULL
mydata1$st019bq01t <- NULL
mydata1$st019cq01t <- NULL
change <- c("GENERAL", 0, "PRE-VOCATIONAL", 1, "VOCATIONAL", 1, "MODULAR", 1)
mydata1 <- replace_value(mydata2 = mydata1, oldname = "iscedo", newname = "vocation", change = change)
mydata1 <- na.omit(mydata1)

# Weight normalization
mydata1 <- normalization_weight(mydata1)
mydata1$mu_escs <- ave(mydata1[c("escs", "w_fsturwt0")], mydata1$cntschid, FUN=function(x) weighted.mean(x$escs, x$w_fsturwt0))[[1]]
mydata1$pct_im <- ave(mydata1[c("immig", "w_fsturwt0")], mydata1$cntschid, FUN=function(x) weighted.mean(x$immig, x$w_fsturwt0))[[1]]
mydata1$type <- mydata1$vocation

# Multilevel for one plausible value
depended <- "pv1scie"
independed <- c("escs", "gender", "type", "mu_escs", "escs*type", "escs*mu_escs", "gender*type", "gender*mu_escs")
random <- c("escs", "gender")
class <- "cntschid"
# Empty model
modmy1a <- multilevel(mydata = mydata1, depended = depended, class = class)
summary.EFECTAS(modmy1a)
# Fix effect
modmy2a <- multilevel(mydata = mydata1, depended = depended, independed = independed, class = class)
summary.EFECTAS(modmy2a)
# Rand effect
modmy3a <- multilevel(mydata = mydata1, depended = depended, independed = independed, random = random, class = class)
summary.EFECTAS(modmy3a)

# Multilevel for combined 10 plausible values
depended <- "scie"
independed <- c("escs", "gender", "type", "mu_escs", "escs*type", "escs*mu_escs", "gender*type", "gender*mu_escs")
random <- c("escs", "gender")
class <- "cntschid"
# Empty model
modmy1b <- multilevel(mydata = mydata1, depended = depended, class = class)
summary.EFECTAS(modmy1b)
# Fix effect
modmy2b <- multilevel(mydata = mydata1, depended = depended, independed = independed, class = class)
summary.EFECTAS(modmy2b)
# Rand effect
modmy3b <- multilevel(mydata = mydata1, depended = depended, independed = independed, random = random, class = class)
summary.EFECTAS(modmy3b)

#************************************
# Vizualization
#************************************
