# All Functions File

# Data Formation
# 5 functions: country_filter; variable_filter; form_data; replace_value; normalization_weight; BIFI_data_pv; BIFI_data
# ---------------------------------------------------------------------------------------------------------------------------


# country_filter function is used to filter country data and form an EdSurvey package object
country_filter <- function(path_root, country){
    if (country == "all"){ 
        # Assigns all country codes. Not recommended
        country <- c("ALB", "ARE", "AUS", "AUT", "BEL", "BGR", "BRA", "CAN", "CHE", "CHL",
                     "COL", "CRI", "CZE", "DEU", "DNK", "DOM", "DZA", "ESP", "EST", "FIN",
                     "FRA", "GBR", "GEO", "GRC", "HKG", "HRV", "HUN", "IDN", "IRL", "ISL",
                     "ISR", "ITA", "JOR", "JPN", "KOR", "KSV", "LBN", "LTU", "LUX", "LVA",
                     "MAC", "MDA", "MEX", "MKD", "MLT", "MNE", "NLD", "NOR", "NZL", "PER",
                     "POL", "PRT", "QAR", "QAT", "QCH",  "QES", "QUC", "QUD", "QUE", "ROU",
                     "RUS", "SGP", "SVK", "SVN", "SWE", "TAP", "THA", "TTO", "TUN", "TUR",
                     "URY", "USA", "VNM")
    }
    new_path_root <- paste0(path_root, "PISA/2015")
    Pisa.data <<- readPISA(path = new_path_root, database = "INT", countries = country)
    return(Pisa.data)
}

# variable_filter function is used to filter variables and formulate data suitable for BIFIsurvey and intsvy packages 
variable_filter <- function(mydata, myvariables){
    must_coll <- c("cnt", "cntschid", "cntstuid", "w_fstuwt", "w_fsturwt1", "w_fsturwt2", "w_fsturwt3", 
                   "w_fsturwt4", "w_fsturwt5", "w_fsturwt6", "w_fsturwt7", "w_fsturwt8", "w_fsturwt9", 
                   "w_fsturwt10", "w_fsturwt11", "w_fsturwt12", "w_fsturwt13", "w_fsturwt14", "w_fsturwt15", 
                   "w_fsturwt16", "w_fsturwt17", "w_fsturwt18", "w_fsturwt19", "w_fsturwt20", "w_fsturwt21", 
                   "w_fsturwt22", "w_fsturwt23", "w_fsturwt24", "w_fsturwt25", "w_fsturwt26", "w_fsturwt27", 
                   "w_fsturwt28", "w_fsturwt29", "w_fsturwt30", "w_fsturwt31", "w_fsturwt32", "w_fsturwt33", 
                   "w_fsturwt34", "w_fsturwt35", "w_fsturwt36", "w_fsturwt37", "w_fsturwt38", "w_fsturwt39", 
                   "w_fsturwt40", "w_fsturwt41", "w_fsturwt42", "w_fsturwt43", "w_fsturwt44", "w_fsturwt45", 
                   "w_fsturwt46", "w_fsturwt47", "w_fsturwt48", "w_fsturwt49", "w_fsturwt50", "w_fsturwt51", 
                   "w_fsturwt52", "w_fsturwt53", "w_fsturwt54", "w_fsturwt55", "w_fsturwt56", "w_fsturwt57", 
                   "w_fsturwt58", "w_fsturwt59", "w_fsturwt60", "w_fsturwt61", "w_fsturwt62", "w_fsturwt63", 
                   "w_fsturwt64", "w_fsturwt65", "w_fsturwt66", "w_fsturwt67", "w_fsturwt68", "w_fsturwt69", 
                   "w_fsturwt70", "w_fsturwt71", "w_fsturwt72", "w_fsturwt73", "w_fsturwt74", "w_fsturwt75", 
                   "w_fsturwt76", "w_fsturwt77", "w_fsturwt78", "w_fsturwt79", "w_fsturwt80")
    all_coll <- c(myvariables, must_coll)
    data.file <- getData(mydata, varnames = all_coll,omittedLevels = FALSE)
    return(data.file)
}

# form_data function to formulate data suitable for use with BIFIsurvey and intsvy packages 
form_data <- function(path_root, mycountry, myvariables){
    country_code <- country_filter(path_root, country = mycountry)
    package_data <- variable_filter(country_code, myvariables)
    return(package_data)
}

# help to clear data like OECD
replace_value <-  function(mydata, oldname, newname = NULL, change){
    ilgis <- length(change)
    if(is.null(newname)){
        newname <- "naujas"
    }
    for(i in 1:ilgis){
        if (i %% 2 == 1){
            mydata[[newname]][mydata[[oldname]] == change[i]] <- as.numeric(change[i+1])
        }
    }
    if(newname == "naujas"){
        mydata[[oldname]] <- mydata[[newname]]
        mydata[[newname]] <- NULL
        newname <- NULL
    }
    if(!is.null(newname)){
        if(oldname != newname){
            mydata[,oldname] <- NULL
        } 
    }
    return(mydata)
}

# weiht normalization
normalization_weight <- function(mydata){
    suma <- sum(mydata[,"w_fstuwt"])
    N <- nrow(mydata)
    mydata$w_fsturwt0 <- mydata$w_fstuwt
    stulpeliai <- c("w_fsturwt0","w_fsturwt1", "w_fsturwt2", "w_fsturwt3", 
                    "w_fsturwt4", "w_fsturwt5", "w_fsturwt6", "w_fsturwt7", "w_fsturwt8", "w_fsturwt9", 
                    "w_fsturwt10", "w_fsturwt11", "w_fsturwt12", "w_fsturwt13", "w_fsturwt14", "w_fsturwt15", 
                    "w_fsturwt16", "w_fsturwt17", "w_fsturwt18", "w_fsturwt19", "w_fsturwt20", "w_fsturwt21", 
                    "w_fsturwt22", "w_fsturwt23", "w_fsturwt24", "w_fsturwt25", "w_fsturwt26", "w_fsturwt27", 
                    "w_fsturwt28", "w_fsturwt29", "w_fsturwt30", "w_fsturwt31", "w_fsturwt32", "w_fsturwt33", 
                    "w_fsturwt34", "w_fsturwt35", "w_fsturwt36", "w_fsturwt37", "w_fsturwt38", "w_fsturwt39", 
                    "w_fsturwt40", "w_fsturwt41", "w_fsturwt42", "w_fsturwt43", "w_fsturwt44", "w_fsturwt45", 
                    "w_fsturwt46", "w_fsturwt47", "w_fsturwt48", "w_fsturwt49", "w_fsturwt50", "w_fsturwt51", 
                    "w_fsturwt52", "w_fsturwt53", "w_fsturwt54", "w_fsturwt55", "w_fsturwt56", "w_fsturwt57", 
                    "w_fsturwt58", "w_fsturwt59", "w_fsturwt60", "w_fsturwt61", "w_fsturwt62", "w_fsturwt63", 
                    "w_fsturwt64", "w_fsturwt65", "w_fsturwt66", "w_fsturwt67", "w_fsturwt68", "w_fsturwt69", 
                    "w_fsturwt70", "w_fsturwt71", "w_fsturwt72", "w_fsturwt73", "w_fsturwt74", "w_fsturwt75", 
                    "w_fsturwt76", "w_fsturwt77", "w_fsturwt78", "w_fsturwt79", "w_fsturwt80")
    mydata[,stulpeliai] <- (mydata[,stulpeliai]/suma)*N
    mydata$w_fstuwt <- NULL
    mydata$nbre <- N
    mydata$somwgt <- suma
    return(mydata)
}

# BIFI_data_pv function for BIFIobject formation when plausible values are combined
BIFI_data_pv <- function(package_data, myvariables){
    data <- package_data
    # user used plausible values
    all_pv_names <- c("math", "read", "scie", "scep", "sced", "scid", "skco", "skpe", "ssph", "ssli", "sses", "flit", "clps")
    my_pv_names <- intersect(myvariables,all_pv_names)
    pv_vars <- my_pv_names
    # create 10 datasets including different sets of plausible values
    dfr <- NULL
    VV <- length(pv_vars)
    Nimp <- 10 # number of plausible values
    for (vv in 1:VV){
        vv1 <- pv_vars[vv]
        ind.vv1 <- which( colnames(data) %in% paste0("pv", 1:Nimp, vv1) )
        dfr2 <- data.frame( "variable"=paste0("PV", vv1), "var_index"=vv,
                            "data_index"=ind.vv1, "impdata_index"=1:Nimp )
        dfr <- rbind( dfr, dfr2 )
    }
    sel_ind <- setdiff( 1:( ncol(data) ), dfr$data_index )
    data0 <- data[, sel_ind ]
    V0 <- ncol(data0)
    newvars <- seq( V0+1, V0+VV )
    datalist <- as.list( 1:Nimp )
    for (ii in 1:Nimp ){
        dat1 <- data.frame( data0, data[, dfr[ dfr$impdata_index==ii, "data_index" ]])
        colnames(dat1)[ newvars ] <- pv_vars
        datalist[[ii]] <- dat1
    }
    # dataset with replicate weights
    datarep <- data[, grep( "w_fsturwt", colnames(data) ) ]
    RR <- ncol(datarep) # number of replicate weights
    # create BIFIE object
    bifieobj <- BIFIEsurvey::BIFIE.data( datalist, wgt=data[, "w_fstuwt"],
                                         wgtrep=datarep, fayfac=1 / RR / ( 1 - .5 )^2 )
    # For PISA: RR=80 and therefore fayfac=1/20=.05
    return(bifieobj)
}

# BIFI_data function for BIFIobject formation without combining plausible values
BIFI_data <- function(package_data){
    data <- package_data
    # dataset with replicate weights
    datarep <- data[, grep( "w_fsturwt", colnames(data) ) ]
    RR <- ncol(datarep) # number of replicate weights
    # create BIFIE object
    bifieobj <- BIFIEsurvey::BIFIE.data( data, wgt=data[, "w_fstuwt"],
                                         wgtrep=datarep, fayfac=1 / RR / ( 1 - .5 )^2)
    # For PISA: RR=80 and therefore fayfac=1/20=.05
    return(bifieobj)
}

# ---------------------------------------------------------------------------------------------------------------------------
# Descriptive statistic
# ---------------------------------------------------------------------------------------------------------------------------


# Frequency table, percentage, missing values
frequency_table <- function(mydata, myvariable, variables, mygroup = NULL, missing_values = FALSE, uniq = FALSE){
    if(missing_values == FALSE){
        if(any(grepl("pv",variables))== TRUE){
            bdat <- BIFI_data(mydata)
        } else {
            bdat <- BIFI_data_pv(mydata, myvariable)
        }
        if(uniq == FALSE){
            lentele <- BIFIE.freq(BIFIEobj = bdat, vars = variables, group = mygroup, se = FALSE) # BIFIEsurvey
            lentele[["stat"]][["perc"]] <- lentele[["stat"]][["perc"]]*100
        } else {
            lentele <- BIFIE.by(BIFIEobj = bdat, vars = variables, userfct = userfct, userparnames = paste0(variables), 
                                group = mygroup, se = FALSE) # BIFIEsurvey
        }
        lentele <- lentele[["stat"]]
    } else {
        if(length(variables) == 1){
            lentele <- summary2(data = Pisa.data,variable = variables, weightVar = "w_fstuwt") # EdSurvey
        } else {
            stop('Missing values can be calculated only for one variable') 
        }
    }
    return(lentele)
}

# Function for number of unique entries. BIFIEsurvey
userfct <- function(X,w){
    pars <- apply( X, 2, FUN = function(vv){
        length( unique(vv))  
    })
    return(pars)
}

# Min, Max, Average, Standard deviation, Median, Quartile for ordinal variables. EdSurvey
descriptive <- function(variables){
    table <- summary2(data = Pisa.data, variable = variables, weightVar = "w_fstuwt")
    return(table)
}

# Percentiles EdSurvey
tile <- function(variables, percent){
    percentiliai <- percentile(variable = variables, percentiles = percent, data = Pisa.data, weightVar = "w_fstuwt")
    return(percentiliai)
}

# Benchmarks only for all plausible values intsvy
ben_marks <- function(mydata, variable, bench){
    colnames(mydata) <- toupper(colnames(mydata))
    variable <- toupper(variable)
    zymes <- pisa2015.ben.pv(pvlabel = variable, by = "CNT", cutoff = bench, data = mydata)
    colnames(mydata) <- tolower(colnames(mydata))
    return(zymes)
}

# ---------------------------------------------------------------------------------------------------------------------------
# Correlation
# ---------------------------------------------------------------------------------------------------------------------------


correlation <- function(mydata, myvariables, variables, group = NULL, method = c("Pearson","Spearman")){
    if((length(variables)==2) & (any(grepl("pv",variables))== TRUE)){
        # EdSurvey
        mycorr <- cor.sdf(x = variables[1],y = variables[2], data = Pisa.data, method = method, weightVar="w_fstuwt")
    } else {
        # BIFIEsurvey
        if(any(grepl("pv",variables))== TRUE){
            bdat <- BIFI_data(mydata)
        } else {
            bdat <- BIFI_data_pv(mydata, myvariables)
        }
        mycorr <- as.list(1:4)
        corr <- BIFIE.correl(bdat,vars = variables, group = group, se = TRUE)
        mycorr[[1]] <- corr[["stat.cor"]]
        mycorr[[2]] <- corr[["cor_matrix"]]
        mycorr[[3]] <- corr[["stat.cov"]]
        mycorr[[4]] <- corr[["cov_matrix"]]
        names(mycorr) <- c("Correlation statistic","Correlation matrix","Covariance statistic","Covariance matrix")
    }
    return(mycorr)
}

# ---------------------------------------------------------------------------------------------------------------------------
# Regression
# ---------------------------------------------------------------------------------------------------------------------------

line_regression <-  function(mydata, myvariables, depended, independed, num_pack){
    formula <- as.formula(paste(depended, paste(independed, collapse=" + "), sep=" ~ "))
    if(num_pack == 1){
        #BIFIEsurvey
        if((any(grepl("pv",depended))== TRUE) | (any(grepl("pv",independed))== TRUE)){
            bdat <- BIFI_data(mydata)
        } else {
            bdat <- BIFI_data_pv(mydata, myvariables)
        }
        regression <- BIFIE.linreg(BIFIEobj = bdat, formula = formula, se = FALSE)
    } else if (num_pack == 2) {
        #EdSurvey
        regression <- lm.sdf(formula = formula, data = Pisa.data, weightVar = "w_fstuwt")
    } else {
        #intsvy
        if((any(grepl("pv",depended))== TRUE) | (any(grepl("pv",independed))== TRUE)){
            regresion <- pisa2015.reg(y = depended, x = independed, by = "cnt", data = mydata)
        } else {
            colnames(mydata) <- toupper(colnames(mydata))
            depended <- toupper(depended)
            independed <- toupper(independed)
            regression <- pisa2015.reg.pv(x = independed, pvlabel = depended, by = "CNT", data = mydata)
            colnames(mydata) <- tolower(colnames(mydata))
        }
    }
    return(regression)
}

log_regression <-  function(mydata, myvariables, depended, independed){
    formula <- as.formula(paste(depended, paste(independed, collapse=" + "), sep=" ~ "))
    #BIFIEsurvey
    if((any(grepl("pv",depended))== TRUE) | (any(grepl("pv",independed))== TRUE)){
        bdat <- BIFI_data(mydata)
    } else {
        bdat <- BIFI_data_pv(mydata, myvariables)
    }
    regression <- BIFIE.logistreg(BIFIEobj = bdat, formula = formula, se = FALSE)
    regression <- regression[["stat"]]
    return(regression)
}

multi_regression <-  function(mydata, depended, independed){
    formula <- as.formula(paste(paste(depended, collapse = "|"), paste(independed, collapse=" + "), sep=" ~ "))
    #EdSurvey
    regression <- mvrlm.sdf(formula = formula, data = Pisa.data, weightVar = "w_fstuwt")
    return(regression)
}

multilevel_no_pv <- function(mydata, depended, independed = NULL, random = NULL, class){
    
    # Form formula
    if(is.null(independed) && is.null(random)){
        dep_in <- paste(depended, 1, sep=" ~ ")
        f_class <-  paste("(",paste(1,class,sep = "|"),")")
        mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
    } else if (is.null(random)){
        dep_in <- (paste(depended, paste(independed, collapse=" + "), sep=" ~ "))
        f_class <-  paste("(",paste(1,class,sep = "|"),")")
        mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
    } else {
        dep_in <- (paste(depended, paste(independed, collapse=" + "), sep=" ~ "))
        f_class <-  paste("(",paste(paste(random, collapse=" + "),class,sep = "||"),")")
        mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
    }
    sink('analysis_output.txt')
    cat("Formula: ")
    cat(paste(deparse(mula, width.cutoff=getOption("width")), collapse="\n"))
    sink()
    
    for(i in 0:80){
        newweight <- as.symbol(paste0("w_fsturwt",i))
        mod <- lmer(mula, data = mydata, REML = FALSE, weights = mydata[[newweight]])
        sink('analysis_output.txt', append=TRUE)
        cat("\n==========================================================\n")
        cat(paste0("Weight ", i, " result\n"))
        cat("==========================================================\n")
        print(summary(mod))
        sink()
        
        fix_temp <- data.frame(keyName=names(lme4::fixef(mod)), value=lme4::fixef(mod), row.names=NULL)
        fix_temp$L <- i
        rand_temp <- as.data.frame(VarCorr(mod))
        rand_temp$var2 <- NULL
        rand_temp$sdcor <- NULL
        intraclass_temp <- data.frame(L = i, Intercept = rand_temp$vcov[1], Residual = rand_temp$vcov[nrow(rand_temp)])
        rand_temp$L <- i
        if (i == 0){
            all_fix_temp <- fix_temp
            all_rand_temp <- rand_temp
            all_intraclass_temp <- intraclass_temp
        } else {
            all_fix_temp <- rbind(all_fix_temp,fix_temp)
            all_rand_temp <- rbind(all_rand_temp, rand_temp)
            all_intraclass_temp <- rbind(all_intraclass_temp,intraclass_temp)
        }
    }
    
    # Information
    N <- mod@devcomp$dims[["N"]]
    Ngroups <- mod@Gp[2]
    pvss <- depended
    groupname <- class
    formformula <- formula
    info <- list(N = N, groupname = groupname, Ngroups = Ngroups, pvss=pvss, formula = formformula)
    
    # Fix effect all weight error
    fix_temp_3 <- split(all_fix_temp,all_fix_temp$L == 0)
    fix_temp_4 <- data.frame(keyName = fix_temp_3[["FALSE"]][["keyName"]],estimate = fix_temp_3[["FALSE"]][["value"]], stat = fix_temp_3[["TRUE"]][["value"]])
    fix_temp_4$vari <- ((fix_temp_4$estimate - fix_temp_4$stat)^2)*(1/20)
    fix_ss <- aggregate(fix_temp_4$vari, by=list(keyName = fix_temp_4$keyName), FUN=sum)
    fix_sestat <- fix_ss$x^0.5
    fix_answer <- data.frame(fix_temp_3[["TRUE"]][["keyName"]],round(fix_temp_3[["TRUE"]][["value"]],2),round(fix_sestat,2))
    colnames(fix_answer) <- c("","Estimate","Std. Error")
    
    # Rand effect all weight error
    rand_temp_3 <- split(all_rand_temp, all_rand_temp$L == 0)
    rand_temp_4 <- data.frame(grp = rand_temp_3[["FALSE"]][["grp"]], covparm = rand_temp_3[["FALSE"]][["var1"]], 
                              estimate = rand_temp_3[["FALSE"]][["vcov"]], stat = rand_temp_3[["TRUE"]][["vcov"]])
    rand_temp_4$vari <- ((rand_temp_4$estimate - rand_temp_4$stat)^2)*(1/20)
    rand_ss <- aggregate(rand_temp_4$vari, by=list(grp = rand_temp_4$grp), FUN=sum)
    rand_sestat <- rand_ss$x^0.5
    rand_answer <- data.frame(rand_temp_3[["TRUE"]][["grp"]], rand_temp_3[["TRUE"]][["var1"]], round(rand_temp_3[["TRUE"]][["vcov"]],2), round(rand_sestat,2))
    colnames(rand_answer) <- c("Groups","Name","Variance","Std. Dev.")
    
    # Intraclass correlation
    all_intraclass_temp$Estimate <- (all_intraclass_temp$Intercept / (all_intraclass_temp$Intercept + all_intraclass_temp$Residual))
    intraclass_temp_3 <- split(all_intraclass_temp, all_intraclass_temp$L == 0)
    intraclass_temp_4 <- data.frame(all = intraclass_temp_3[["FALSE"]][["Estimate"]], zero = intraclass_temp_3[["TRUE"]][["Estimate"]])
    intraclass_temp_4$vari <- ((intraclass_temp_4$all - intraclass_temp_4$zero)^2)*(1/20)
    intraclass_ss <- sum(intraclass_temp_4$vari)
    intraclass_sestat <- intraclass_ss^0.5
    intraclass_corr <- data.frame(round(intraclass_temp_3[["TRUE"]][["Estimate"]],2), round(intraclass_sestat,2))
    colnames(intraclass_corr) <- c("Estimate","Std. Error")
    rezz <- list(info = info, fix = fix_answer, rand = rand_answer, intraclass = intraclass_corr)
    rm(mod, fix_temp, all_fix_temp, fix_temp_3, fix_temp_4, fix_ss, fix_sestat, fix_answer,
       rand_temp, all_rand_temp, rand_temp_3, rand_temp_4, rand_ss, rand_sestat, rand_answer,
       intraclass_temp, all_intraclass_temp, intraclass_temp_3, intraclass_temp_4, intraclass_ss, intraclass_sestat, intraclass_corr)
    return(rezz)
}

multilevel_pv <- function(mydata, depended, independed = NULL, random = NULL, class){
    
    for(j in 1:10){
        plausible <- paste0("pv", j, depended)
        # Form formula
        if(is.null(independed) && is.null(random)){
            dep_in <- paste(plausible, 1, sep=" ~ ")
            f_class <-  paste("(",paste(1,class,sep = "|"),")")
            mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
        } else if (is.null(random)){
            dep_in <- (paste(plausible, paste(independed, collapse=" + "), sep=" ~ "))
            f_class <-  paste("(",paste(1,class,sep = "|"),")")
            mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
        } else {
            dep_in <- (paste(plausible, paste(independed, collapse=" + "), sep=" ~ "))
            f_class <-  paste("(",paste(paste(random, collapse=" + "),class,sep = "||"),")")
            mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
        }
        if(j==1){
            sink('analysis_output.txt')
            cat(paste0(j, " plausible value\n"))
            cat("Formula: ")
            cat(paste(deparse(mula, width.cutoff=getOption("width")), collapse="\n"))
            sink()
        } else {
            sink('analysis_output.txt', append=TRUE)
            cat("\n\n~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~\n")
            cat(paste0(j, " plausible value\n"))
            cat("Formula: ")
            cat(paste(deparse(mula, width.cutoff=getOption("width")), collapse="\n"))
            sink()
        }
        
        
        for(i in 0:80){
            newweight <- as.symbol(paste0("w_fsturwt",i))
            mod <- lmer(formula = mula, data = mydata, REML = FALSE, weights = mydata[[newweight]])
            sink('analysis_output.txt', append=TRUE)
            cat("\n==========================================================\n")
            cat(paste0("PV ", j, " value weight ", i, " result\n"))
            cat("==========================================================\n")
            print(summary(mod))
            sink()
            
            fix_temp <- data.frame(keyName=names(lme4::fixef(mod)), value=lme4::fixef(mod), L=i, pv=lme4::fixef(mod), row.names=NULL)
            colnames(fix_temp) <- c("keyName","value","L",plausible)
            rand_temp <- as.data.frame(VarCorr(mod))
            rand_temp$var2 <- NULL
            rand_temp$sdcor <- NULL
            rand_temp$L <- i
            rand_temp$pv <- rand_temp$vcov
            colnames(rand_temp) <- c("grp","var1","vcov","L",plausible)
            pv <- (rand_temp$vcov[1] / (rand_temp$vcov[1] + rand_temp$vcov[nrow(rand_temp)]))
            intraclass_temp <- data.frame(L = i, pv)
            colnames(intraclass_temp) <- c("L", plausible)
            if (i == 0 ){
                all_fix_temp_w <- fix_temp
                all_rand_temp_w <- rand_temp
                all_intraclass_temp_w <- intraclass_temp
            } else {
                all_fix_temp_w <- rbind(all_fix_temp_w,fix_temp)
                all_rand_temp_w <- rbind(all_rand_temp_w, rand_temp)
                all_intraclass_temp_w <- rbind(all_intraclass_temp_w,intraclass_temp)
            }
        }
        if (j == 1){
            all_fix_temp <- all_fix_temp_w
            all_rand_temp <- all_rand_temp_w
            all_intraclass_temp <- all_intraclass_temp_w
            cat("\n|*")
        } else {
            all_fix_temp <- cbind(all_fix_temp,all_fix_temp_w[[ncol(all_fix_temp_w)]])
            colnames(all_fix_temp)[ncol(all_fix_temp)] <- plausible
            all_rand_temp <- cbind(all_rand_temp, all_rand_temp_w[[ncol(all_rand_temp_w)]])
            colnames(all_rand_temp)[ncol(all_rand_temp)] <- plausible
            all_intraclass_temp <- cbind(all_intraclass_temp, all_intraclass_temp_w[[ncol(all_intraclass_temp_w)]])
            colnames(all_intraclass_temp)[ncol(all_intraclass_temp)] <- plausible
            cat("*")
        }
        if(j==10) cat("|\n")
    }
    
    # Information
    N <- mod@devcomp$dims[["N"]]
    Ngroups <- mod@Gp[2]
    pvss <- depended
    groupname <- class
    if(is.null(independed) && is.null(random)){
        dep_in <- paste(depended, 1, sep=" ~ ")
        f_class <-  paste("(",paste(1,class,sep = "|"),")")
        form_mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
    } else if (is.null(random)){
        dep_in <- (paste(depended, paste(independed, collapse=" + "), sep=" ~ "))
        f_class <-  paste("(",paste(1,class,sep = "|"),")")
        form_mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
    } else {
        dep_in <- (paste(depended, paste(independed, collapse=" + "), sep=" ~ "))
        f_class <-  paste("(",paste(paste(random, collapse=" + "),class,sep = "||"),")")
        form_mula <- as.formula(paste(c(dep_in,f_class),collapse=" + "))
    }
    info <- list(N = N, groupname = groupname, Ngroups = Ngroups, pvss = pvss, formula = form_mula)
    
    # Fix effect all weight error
    fix_temp_3 <- split(all_fix_temp,all_fix_temp$L == 0)
    fix_temp_3_all <- data.frame(fix_temp_3[["FALSE"]])
    fix_temp_3_all$value <- NULL
    fix_temp_3_zero <- data.frame(fix_temp_3[["TRUE"]])
    fix_temp_3_zero$value <- rowMeans(fix_temp_3_zero[,4:ncol(fix_temp_3_zero)])
    for(i in 1:10){
        stulpelis <- 3 + i
        if(i==1){
            mesvar <- (fix_temp_3_zero$value - fix_temp_3_zero[,stulpelis])^2
            fa <- paste0("fin",i)
            fb <- fa
            va <- paste0("var",i)
            vb <- va
        } else {
            mesvar <- mesvar + ((fix_temp_3_zero$value - fix_temp_3_zero[,stulpelis])^2)
            fa <- paste0("fin",i)
            fb <- c(fb,fa)
            va <- paste0("var",i)
            vb <- c(vb,va)
        }
    }
    fix_temp_3_zero$mesvar <- mesvar/9
    colnames(fix_temp_3_zero) <- c("keyname", "stat","L", fb, "mesvar")
    fix_temp_4 <- data.frame(fix_temp_3_all,stat=fix_temp_3_zero$stat,fix_temp_3_zero[,fb],mesvar=fix_temp_3_zero$mesvar)
    for(i in 1:10){
        a <- paste0("pv",i,depended)
        b <- paste0("fin",i)
        ab <- paste0("var",i)
        fix_temp_4[,ab] <- ((fix_temp_4[,a] - fix_temp_4[,b])^2)*(1/20)
    }
    fix_ss <- aggregate(fix_temp_4[,vb], by=list(keyName = fix_temp_4$keyName), FUN=sum)
    fix_samp <- rowMeans(fix_ss[,2:ncol(fix_ss)])
    fix_finvar <- (fix_samp + (1.2*fix_temp_3_zero$mesvar))
    fix_sestat <- fix_finvar^0.5
    fix_answer <- data.frame(fix_temp_3_zero$keyname,round(fix_temp_3_zero$stat,2),round(fix_sestat,2))
    colnames(fix_answer) <- c("","Estimate","Std. Error")
    
    # Rand effect all weight error
    rand_temp_3 <- split(all_rand_temp, all_rand_temp$L == 0)
    rand_temp_3_all <- data.frame(rand_temp_3[["FALSE"]])
    rand_temp_3_all$vcov <- NULL
    rand_temp_3_zero <- data.frame(rand_temp_3[["TRUE"]])
    rand_temp_3_zero$vcov <- rowMeans(rand_temp_3_zero[,5:ncol(rand_temp_3_zero)])
    for(i in 1:10){
        stulpelis <- 4 + i
        if(i==1){
            mesvar <- (rand_temp_3_zero$vcov - rand_temp_3_zero[,stulpelis])^2
        } else {
            mesvar <- mesvar + ((rand_temp_3_zero$vcov - rand_temp_3_zero[,stulpelis])^2)
        }
    }
    rand_temp_3_zero$mesvar <- mesvar/9
    colnames(rand_temp_3_zero) <- c("grp", "var1", "vcov", "L", fb, "mesvar")
    rand_temp_4 <- data.frame(rand_temp_3_all,vcov=rand_temp_3_zero$vcov,rand_temp_3_zero[,fb],mesvar=rand_temp_3_zero$mesvar)
    for(i in 1:10){
        a <- paste0("pv",i,depended)
        b <- paste0("fin",i)
        ab <- paste0("var",i)
        rand_temp_4[,ab] <- ((rand_temp_4[,a] - rand_temp_4[,b])^2)*(1/20)
    }
    rand_ss <- aggregate(rand_temp_4[,vb], by=list(grp = rand_temp_4$grp), FUN=sum)
    rand_samp <- rowMeans(rand_ss[,2:ncol(rand_ss)])
    rand_finvar <- (rand_samp + (1.2*rand_temp_3_zero$mesvar))
    rand_sestat <- rand_finvar^0.5
    rand_answer <- data.frame(rand_temp_3_zero$grp, rand_temp_3_zero$var1, round(rand_temp_3_zero$vcov,2), round(rand_sestat,2))
    colnames(rand_answer) <- c("Groups","Name","Variance","Std. Dev.")
    
    # Intraclass correlation
    intraclass_temp_3 <- split(all_intraclass_temp, all_intraclass_temp$L == 0)
    intraclass_temp_3_all <- data.frame(intraclass_temp_3[["FALSE"]])
    intraclass_temp_3_zero <- data.frame(intraclass_temp_3[["TRUE"]])
    intraclass_temp_3_zero$stat <- rowMeans(intraclass_temp_3_zero[,2:ncol(intraclass_temp_3_zero)])
    for(i in 1:10){
        stulpelis <- 1 + i
        if(i==1){
            mesvar <- (intraclass_temp_3_zero$stat - intraclass_temp_3_zero[,stulpelis])^2
        } else {
            mesvar <- mesvar + ((intraclass_temp_3_zero$stat - intraclass_temp_3_zero[,stulpelis])^2)
        }
    }
    intraclass_temp_3_zero$mesvar <- mesvar/9
    colnames(intraclass_temp_3_zero) <- c("L", fb, "stat", "mesvar")
    intraclass_temp_4 <- data.frame(intraclass_temp_3_all,stat=intraclass_temp_3_zero$stat,intraclass_temp_3_zero[,fb],mesvar=intraclass_temp_3_zero$mesvar)
    for(i in 1:10){
        a <- paste0("pv",i,depended)
        b <- paste0("fin",i)
        ab <- paste0("var",i)
        intraclass_temp_4[,ab] <- ((intraclass_temp_4[,a] - intraclass_temp_4[,b])^2)*(1/20)
    }
    intraclass_ss <- colSums(intraclass_temp_4[,vb])
    intraclass_samp <- mean(intraclass_ss)
    intraclass_finvar <- (intraclass_samp + (1.2*intraclass_temp_3_zero$mesvar))
    intraclass_sestat <- intraclass_finvar^0.5
    intraclass_corr <- data.frame(round(intraclass_temp_3_zero$stat,2), round(intraclass_sestat,2))
    colnames(intraclass_corr) <- c("Estimate","Std. Error")
    
    #model answer
    rezz <- list(info = info, fix = fix_answer, rand = rand_answer, intraclass = intraclass_corr)
    rm(mod, fix_temp, all_fix_temp, all_fix_temp_w, fix_temp_3, fix_temp_3_all, fix_temp_3_zero, fix_temp_4, fix_ss, fix_samp, fix_sestat, fix_finvar, fix_answer,
       rand_temp, all_rand_temp, all_rand_temp_w, rand_temp_3, rand_temp_3_all, rand_temp_3_zero, rand_temp_4, rand_ss, rand_samp, rand_sestat, rand_finvar, rand_answer,
       intraclass_temp, all_intraclass_temp, all_intraclass_temp_w, intraclass_temp_3, intraclass_temp_3_all, intraclass_temp_3_zero, intraclass_temp_4, intraclass_ss, 
       intraclass_samp, intraclass_finvar, intraclass_sestat, intraclass_corr)
    return(rezz)
}

multilevel <- function(mydata, depended, independed = NULL, random = NULL, class){
    if(any(grepl("pv",depended))== TRUE){
        rezzz <- multilevel_no_pv(mydata = mydata, depended = depended, independed = independed, random = random, class = class)
    } else {
        rezzz <- multilevel_pv(mydata = mydata, depended = depended, independed = independed, random = random, class = class)
    }
    return(rezzz)
}

printList <- function(list) {
    cat("Linear mixed model fit by maximum likelihood.\n")
    cat("Formula: ")
    cat(paste(deparse(list$info[["formula"]], width.cutoff=getOption("width")), collapse="\n"))
    if(any(grepl("pv",list$info[["pvss"]]))==TRUE){
        cat("\nStandard errors were calculated for all replicate weights.\n")
    } else {
        cat("\nComputation of final estimates and their respective standard errors.\n")
    }
    cat("\nRandom effects:\n")
    print(list[["rand"]], row.names = FALSE, na.print = "")
    cat("\nNumber of object: ", paste0(list$info[["N"]],", groups: "), paste0(list$info[["groupname"]],", "),list$info[["Ngroups"]],"\n "  )
    cat("\nFixed effects:\n")
    print(list[["fix"]], row.names = FALSE, na.print = "")
    cat("\nIntraclass correlation:\n")
    print(list[["intraclass"]], row.names = FALSE, na.print = "")
}

summary.EFECTAS <- function (list) {
    sink('result_output.txt')
    printList(list)
    sink()
    printList(list)
    cat("\nAll models results you can find in file analysis_output.txt\n")
    cat("This output you can find in file result_output.txt")
}
